@@ -427,20 +427,25 @@ get_changed_events (ECalBackendExchange *cbex)
 	status = e2k_result_iter_free (iter);
 
 	if (!SOUP_STATUS_IS_SUCCESSFUL (status)) {
-		if (!since)
+		if (!since) {
+			e_cal_backend_exchange_cache_lock (cbex);
 			e_cal_backend_exchange_cache_sync_end (cbex);
+			e_cal_backend_exchange_cache_unlock (cbex);
+		}
 
 		g_ptr_array_free (hrefs, TRUE);
 		g_hash_table_destroy (modtimes);
 		g_hash_table_destroy (attachments);
 		g_mutex_unlock (cbexc->priv->mutex);
+		g_object_unref (cbexc);
 		return status;
 	}
 
-	e_cal_backend_exchange_cache_lock (cbex);
-	if (!since)
+	if (!since) {
+		e_cal_backend_exchange_cache_lock (cbex);
 		e_cal_backend_exchange_cache_sync_end (cbex);
-	e_cal_backend_exchange_cache_unlock (cbex);
+		e_cal_backend_exchange_cache_unlock (cbex);
+	}
 
 	if (!hrefs->len) {
 		g_ptr_array_free (hrefs, TRUE);
@@ -448,6 +453,7 @@ get_changed_events (ECalBackendExchange *cbex)
 		g_hash_table_destroy (attachments);
 		cbexc->priv->is_loaded = TRUE;
 		g_mutex_unlock (cbexc->priv->mutex);
+		g_object_unref (cbexc);
 		return SOUP_STATUS_OK;
 	}
 
@@ -502,6 +508,7 @@ get_changed_events (ECalBackendExchange *cbex)
 		g_hash_table_destroy (modtimes);
 		g_hash_table_destroy (attachments);
 		g_mutex_unlock (cbexc->priv->mutex);
+		g_object_unref (cbexc);
 		return status;
 	}
 	if (!hrefs->len) {
@@ -510,6 +517,7 @@ get_changed_events (ECalBackendExchange *cbex)
 		g_hash_table_destroy (attachments);
 		cbexc->priv->is_loaded = TRUE;
 		g_mutex_unlock (cbexc->priv->mutex);
+		g_object_unref (cbexc);
 		return SOUP_STATUS_OK;
 	}
 
@@ -517,6 +525,7 @@ get_changed_events (ECalBackendExchange *cbex)
 	ctx = exchange_account_get_context (cbex->account);
 	if (!ctx) {
 		g_mutex_unlock (cbexc->priv->mutex);
+		g_object_unref (cbexc);
 		/* This either means we lost connection or we are in offline mode */
 		return SOUP_STATUS_CANT_CONNECT;
 	}
@@ -546,6 +555,8 @@ get_changed_events (ECalBackendExchange *cbex)
 		cbexc->priv->is_loaded = TRUE;
 
 	g_mutex_unlock (cbexc->priv->mutex);
+	g_object_unref (cbexc);
+
 	return status;
 }
 
@@ -560,7 +571,7 @@ notify_changes (E2kContext *ctx, const gchar *uri,
 	g_return_if_fail (E_IS_CAL_BACKEND_EXCHANGE (ecalbex));
 	g_return_if_fail (uri != NULL);
 
-	get_changed_events (ecalbex);
+	get_changed_events (g_object_ref (ecalbex));
 
 }
 
@@ -599,11 +610,12 @@ open_calendar (ECalBackendSync *backend, EDataCal *cal,
 			E2K_CONTEXT_OBJECT_REMOVED, 30,
 			notify_changes, backend);
 
-	thread = g_thread_create ((GThreadFunc) get_changed_events, E_CAL_BACKEND_EXCHANGE (backend), FALSE, &error);
+	thread = g_thread_create ((GThreadFunc) get_changed_events, g_object_ref (backend), FALSE, &error);
 	if (!thread) {
 		g_warning (G_STRLOC ": %s", error->message);
 		g_propagate_error (perror, EDC_ERROR_EX (OtherError, error->message));
 		g_error_free (error);
+		g_object_unref (backend);
 	}
 }
 
@@ -612,7 +624,7 @@ refresh_calendar (ECalBackendSync *backend, EDataCal *cal, GError **perror)
 {
 	g_return_if_fail (E_IS_CAL_BACKEND_EXCHANGE (backend));
 
-	get_changed_events (E_CAL_BACKEND_EXCHANGE (backend));
+	get_changed_events (g_object_ref (backend));
 }
 
 struct _cb_data {
@@ -1390,8 +1402,12 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 	   the server, so that we don't lose any detached instances */
 	if (ecomp->icomp && mod == CALOBJ_MOD_THIS && remove) {
 		icalcomponent_add_component (cbdata->vcal_comp, icalcomponent_new_clone (ecomp->icomp));
-	} else if (!remove && mod == CALOBJ_MOD_THIS)
+	}
+
+	if (!remove && mod == CALOBJ_MOD_THIS) {
+		icalcomponent_add_component (cbdata->vcal_comp, icalcomponent_new_clone (ecomp->icomp));
 		icalcomponent_add_component (cbdata->vcal_comp, real_icalcomp);
+	}
 
 	for (l = ecomp->instances; l != NULL; l = l->next) {
 		icalcomponent *icomp = l->data;
@@ -1919,7 +1935,7 @@ book_resource (ECalBackendExchange *cbex,
 	e_cal_component_get_uid (E_CAL_COMPONENT (comp), &uid);
 	sanitized_uid = g_strdup (uid);
 	g_strdelimit (sanitized_uid, " /'\"`&();|<>$%{}!\\:*?#@", '_');
-	href = g_strdup_printf ("%s%s%s.EML", cal_uri [strlen (cal_uri) - 1] == '/' ? "" : "/", cal_uri, sanitized_uid);
+	href = g_strdup_printf ("%s%s%s.EML", cal_uri, cal_uri [strlen (cal_uri) - 1] == '/' ? "" : "/", sanitized_uid);
 	g_free (sanitized_uid);
 
 	e_cal_backend_exchange_cache_lock (cbex);
