@@ -2063,7 +2063,10 @@ build_msg ( ECalBackendExchange *cbex, ECalComponent *comp, const gchar *subject
 		if (!strncmp ((gchar *)l->data, "file://", 7)) {
 			fname = g_filename_from_uri ((gchar *)l->data, NULL, NULL);
 			filename = g_path_get_basename (fname);
-			mime_filename = g_strdup (filename + strlen (uid) + 1);
+			if (g_str_has_prefix (filename, uid))
+				mime_filename = g_strdup (filename + strlen(uid) + 1);
+			else
+				mime_filename = g_strdup (filename);
 			g_free (filename);
 			attach_file = fname;
 		} else {
